import random
from datetime import datetime, timedelta


class Scheduler:
    def __init__(self, sheets):
        self.today = []
        for sheet in sheets:
            if sheet.date is None or sheet.date <= datetime.today().date():
                self.today.append(sheet)
        random.shuffle(self.today)

    def schedule(self, sheet, confidence):
        if confidence == "r":
            self.today.insert(random.randrange(len(self.today) + 1), sheet)
            sheet.interval = 1
        else:
            factor = {"e": 3, "n": 2, "h": 1}[confidence] * random.uniform(0.9, 1.1)
            sheet.interval = round(sheet.interval * factor, 2)
            sheet.date = datetime.today().date() + timedelta(days=int(sheet.interval))



# import sys
import textwrap

import click
from getkey import getkey
from ruamel.yaml import YAML

from .binders import binders
from .scheduler import Scheduler
from .term import term

yaml = YAML()


def read(name):
    if not name.endswith(".yaml"):
        name += ".yaml"
    with open(name, "r") as f:
        # try:
        data = yaml.load(f)
        # except yaml.scanner.ScannerError as e:
        #     print("=" * 30 + " Failed to read file " + "=" * 30)
        #     print(e)
        #     sys.exit(1)
    assert "binder" in data, "Missing binder"
    assert data["binder"] in binders, "Invalid binder"
    assert "sheets" in data, "Missing sheets"
    # print(data)
    binder = binders[data["binder"]](data["sheets"])
    return data, binder


def write(name, data):
    if not name.endswith(".yaml"):
        name += ".yaml"
    with open(name, "w") as f:
        yaml.dump(data, f)


@click.group()
def cli():
    pass


def getnum():
    key = getkey()
    while key not in "renh":
        key = getkey()
    return key


def printwrap(text, centery=False):
    width = 3 * term.width // 5
    lines = []
    for line in text.split("\n"):
        lines += textwrap.wrap(line, width)
    if centery:
        print(term.move((term.height - len(lines)) // 2 - 2, 0))
    for line in lines:
        print(line.center(term.width))


@cli.command()
@click.argument("name")
def rehearse(name):
    data, binder = read(name)
    scheduler = Scheduler(binder.sheets)

    with term.fullscreen():
        while scheduler.today:
            sheet = scheduler.today.pop()
            sheet.print_front(lambda text: printwrap(text, centery=True))
            getkey()
            sheet.print_back(lambda text: printwrap(text, centery=False))
            confidence = getnum()
            scheduler.schedule(sheet, confidence)
            print(term.clear())

    write(name, data)
    print("All done for today :D")


if __name__ == "__main__":
    cli()



from ..sheets import Person
from .binder import Binder


class People(Binder):
    def __init__(self, data):
        self.data = data
        self.sheets = []
        for person in data:
            assert "name" in person, "Missing name: {}".format(person)
            self.sheets.append(Person(person))



class Sheet:
    def __init__(self, data):
        self._data = data

    @property
    def date(self):
        return self._data.get("next")

    @date.setter
    def date(self, date):
        self._data["next"] = date

    @property
    def interval(self):
        return self._data.get("interval", 1)

    @interval.setter
    def interval(self, interval):
        self._data["interval"] = interval

    def print_front(self, printwrap):
        raise NotImplementedError()

    def print_back(self, printwrap):
        raise NotImplementedError()



import base64
import os
import subprocess
import sys
import tempfile

from .term import term


def get_latex_image(latex, width=120, height="auto", color=(22, 22, 22)):
    doc = "\n".join(
        [
            r"\documentclass[10pt]{article}",
            r"\usepackage{geometry}",
            r"\geometry{paperwidth=15cm, paperheight=4cm, margin=0cm}",
            r"\usepackage{amsmath,amsfonts,amsthm,pagecolor,xcolor}",
            r"\begin{document}",
            r"\definecolor{bgcolor}{RGB}{%d,%d,%d}" % color,
            r"\pagecolor{bgcolor}\color{white}",
            r"\begin{center}",
            latex,
            r"\end{center}",
            r"\end{document}",
        ]
    )

    workdir = tempfile.mkdtemp()
    texfile = os.path.join(workdir, "texput.tex")
    with open(texfile, "w") as f:
        f.write(doc)

    subprocess.check_output(["pdflatex", "-output-directory=" + workdir, texfile])
    pdffile = os.path.join(workdir, "texput.pdf")
    pngfile = os.path.join(workdir, "texput.png")
    subprocess.check_output(
        ["convert", "-quiet", "-density", "300", pdffile, "-quality", "90", pngfile]
    )
    with open(pngfile, "rb") as f:
        png = f.read()

    IMAGE_CODE = "\033]1337;File=name={name};inline={inline};size={size};width={width};height={height};preserveAspectRatio={preserve_aspect_ratio}:{base64_img}\a"
    data = {
        "name": base64.b64encode("Unnamed file".encode("utf-8")).decode("ascii"),
        "inline": 1,
        "size": len(png),
        "base64_img": base64.b64encode(png).decode("ascii"),
        "width": width,
        "height": height,
        "preserve_aspect_ratio": 1,
    }
    s = IMAGE_CODE.format(**data).encode("ascii")
    return b" " * ((term.width - 120) // 2) + s


def print_image(image):
    sys.stdout.buffer.write(image)
    print()



from .sheet import Sheet


class Person(Sheet):
    def __init__(self, person):
        self.person = person
        super().__init__(person)

    def print_front(self, printwrap):
        printwrap(self.person["name"])

    def print_back(self, printwrap):
        if "dates" in self.person:
            printwrap(self.person["dates"])
        if "topics" in self.person:
            printwrap(self.person["topics"])



from ..sheets import Quote
from .binder import Binder


class Quotes(Binder):
    def __init__(self, data):
        self.data = data
        self.sheets = []
        for work in data:
            assert "quotes" in work, "Missing quotes for `{}`".format(repr(work))
            for quote in work["quotes"]:
                assert "quote" in quote, "Missing quote for `{}`".format(repr(work))
                self.sheets.append(Quote(work, quote))



from ..sheets import MathSheet
from .binder import Binder


class Math(Binder):
    def __init__(self, data):
        self.data = data
        self.sheets = []
        for sheet in data:
            assert "name" in sheet, "Missing name for `{}`".format(repr(sheet))
            assert "statement" in sheet, "Missing statement for `{}`".format(
                repr(sheet)
            )
            self.sheets.append(MathSheet(sheet))



from .sheet import Sheet
from ..latex import get_latex_image, print_image


class MathSheet(Sheet):
    def __init__(self, sheet):
        self.sheet = sheet
        self.latex_image = None
        super().__init__(sheet)

    def print_front(self, printwrap):
        printwrap(self.sheet["name"])
        latex = self.sheet["statement"]
        if "proof" in self.sheet:
            latex += r"\begin{proof}" + self.sheet["proof"] + r"\end{proof}"
        self.latex_image = get_latex_image(latex)
        print()

    def print_back(self, printwrap):
        print_image(self.latex_image)
        return ""



import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="repertoire",
    version="0.0.2",
    author="Alex Gajewski",
    author_email="agajews@gmail.com",
    description="Spaced repetition in plain text",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/agajews/repertoire",
    packages=setuptools.find_packages(),
    install_requires=["blessings", "click", "getkey"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["repertoire=repertoire.repertoire:cli"]},
)



from .sheet import Sheet


class Quote(Sheet):
    def __init__(self, work, quote):
        self.work = work
        self.quote = quote
        super().__init__(quote)

    def print_front(self, printwrap):
        printwrap(self.quote["quote"])

    def print_back(self, printwrap):
        back = "-- "
        if "title" in self.work:
            back += self.work["title"] + "; "
        if "author" in self.work:
            back += self.work["author"] + " "
        if "date" in self.work:
            back += "(" + str(self.work["date"]) + ")"
        printwrap(back)
